Attribute VB_Name = "MapIt4Meetings"
Public Sub MapCalendarLocation()

'==============================================================
' Description: Outlook macro to get driving instructions
'              to meeting locations.
'
' author : Robert Sparnaaij
' version: 1.0
' website: https://www.howto-outlook.com/howto/mapit4meetings.htm
'==============================================================


'====================READ AND EDIT IF NEEDED==============

  'Select your mapping service by setting the number
  MappingService = 1
  ' 1 = Bing Maps
  ' 2 = Google Maps
  ' 3 = OpenStreetMap
  
  'For this macro to work, it is important that you type
  'the address in the query format required by the mapping
  'service that you selected.
  'This address has to be typed in the "Location" field of
  'the appointment or meeting request.
  
  'Select if you want driving directions.
  'If you set this to True, you must also specify
  'your starting location.
  DrivingDirections = False

  'Specify your starting location
  'Make sure you write your address between double quotes.
  'Example: street = "My Street Name 12"
  addr_street = ""
  addr_city = ""
  addr_state = ""
  addr_zipcode = ""
  addr_country = ""
  
'===================STOP EDITING==========================
  
  Dim FromString As String
  FromString = addr_street & ", " & addr_city & ", " & addr_state & ", " & addr_zipcode & ", " & addr_country
    
  Dim obj As Object
  Dim Appointment As Outlook.AppointmentItem
  Set obj = Application.ActiveWindow
  If TypeOf obj Is Outlook.Inspector Then
    Set obj = obj.CurrentItem
  Else
    Set obj = obj.Selection(1)
  End If

  If TypeOf obj Is Outlook.AppointmentItem Then
    Set Appointment = obj
    Dim LocationString As String
    LocationString = Appointment.Location

    If DrivingDirections = False Then
      Select Case MappingService
        Case 1
          NavigationString = "https://bing.com/maps/?where1=" & URLEncode(LocationString, True)
        Case 2
          NavigationString = "https://www.google.com/maps/search/?api=1&query=" & URLEncode(LocationString, True)
        Case 3
          NavigationString = "https://www.openstreetmap.org/search?query=" & URLEncode(LocationString)
        Case Else
          MsgBox ("No valid mapping service selected.")
        End Select

    ElseIf DrivingDirections = True Then
      Select Case MappingService
        Case 1
          NavigationString = "https://bing.com/maps/?rtp=adr." & _
          URLEncode(FromString, True) & "~adr." & URLEncode(LocationString, True)
        Case 2
          NavigationString = "https://www.google.com/maps/dir/?api=1&origin=" & _
          URLEncode(FromString, True) & "&destination=" & URLEncode(LocationString, True)
        Case 3
          NavigationString = "https://www.openstreetmap.org/directions?from=" & _
           URLEncode(FromString) & "&to=" & URLEncode(LocationString)
        Case Else
          MsgBox ("No valid mapping service selected.")
      End Select

    Else
      MsgBox ("DrivingDirections has not been specified properly.")
      Exit Sub
    End If
    
    Set WshShell = CreateObject("WScript.Shell")
    WshShell.Run NavigationString
    Set WshShell = Nothing
  
  Else
    MsgBox ("No Calendar item selected")
    Exit Sub
  End If
End Sub

Public Function URLEncode(StringVal As String, Optional SpaceAsPlus As Boolean = False) As String

  'URLEncode function taken from
  'https://stackoverflow.com/questions/218181/how-can-i-url-encode-a-string-in-excel-vba/24301379

  Dim StringLen As Long: StringLen = Len(StringVal)

  If StringLen > 0 Then
    ReDim result(StringLen) As String
    Dim i As Long, CharCode As Integer
    Dim Char As String, Space As String

    If SpaceAsPlus Then Space = "+" Else Space = "%20"

    For i = 1 To StringLen
      Char = Mid$(StringVal, i, 1)
      CharCode = Asc(Char)
      Select Case CharCode
        Case 97 To 122, 65 To 90, 48 To 57, 45, 46, 95, 126
          result(i) = Char
        Case 32
          result(i) = Space
        Case 0 To 15
          result(i) = "%0" & Hex(CharCode)
        Case Else
          result(i) = "%" & Hex(CharCode)
      End Select
    Next i
    URLEncode = Join(result, "")
  End If
End Function